/*
 *
 *	File: CmnLog.h 
 *  Desc: Functions and macros for diagnostic (TRACE, ...)
 *  Authors: MrAndrew, Swap
 *  Date:   23.12.2002
 *  Update: 31.12.2002
 *
 */

#ifndef _CMNLOG_H_
	#define _CMNLOG_H_	
	//	
	// Standard tracing with logging
	COMMON_API	void	cmnTraceW(LPCWSTR lpcwszFormat, ...);					
	COMMON_API	void	cmnTraceA(LPCSTR lpcszFormat, ...);						
	// Tracing with logging and decoding error code
	COMMON_API	void	cmnTraceErrW(LPCWSTR lpcwszMsg, HRESULT hr);			
	COMMON_API	void	cmnTraceErrA(LPCSTR lpcszMsg, HRESULT hr);			
	// Throw exception with logging
	COMMON_API	void	cmnThrowW(LPCWSTR lpcwszFormat, ...);	
	COMMON_API	void	cmnThrowA(LPCSTR lpcszFormat, ...);	
	// Throw exception with logging and decoding error code
	COMMON_API	void	cmnThrowErrW(LPCWSTR lpcwszFormat, HRESULT hr);					
	COMMON_API	void	cmnThrowErrA(LPCSTR lpcszFormat, HRESULT hr);						
	// Display message box (standard Windows message box)
	COMMON_API	void	cmnMessageBoxW(LPCWSTR lpcwszCaption, UINT uType);	
	COMMON_API	void	cmnMessageBoxA(LPCSTR lpcszCaption, UINT uType);		
	// Calls theLog.SavePosition()
	COMMON_API	void	cmnSavePosW(LPCWSTR	lpcwszFile, DWORD	dwLine);	
	COMMON_API	void	cmnSavePosA(LPCSTR	lpcszFile, DWORD	dwLine);			
	//
	// Macros
	// If TRACE macro is already defined, undef it	
	#ifdef TRACE
		#undef TRACE
	#endif // TRACE
	// If THROW macro is already defined, undef it	
	#ifdef THROW
		#undef THROW
	#endif // THROW	
	//
	// HUMUS
	#define WIDEN2(x) L ## x
	#define WIDEN(x) WIDEN2(x)
	#define __WFILE__ WIDEN(__FILE__)
	//
	// If the user wants to trace specific ANSI or UNICODE message
	// These macros are always defined
	#define TRACEA		cmnSavePosA(__FILE__, __LINE__); cmnTraceA			// ANSI verze
	#define TRACEW		cmnSavePosW(__WFILE__, __LINE__); cmnTraceW			// UNICODE verze
	#define THROWA		cmnSavePosA(__FILE__, __LINE__); cmnThrowA			// ANSI verze
	#define THROWW		cmnSavePosW(__WFILE__, __LINE__); cmnThrowW			// UNICODE verze
	#define ERRA		cmnSavePosA(__FILE__, __LINE__); cmnTraceErrA		// ANSI verze
	#define ERRW		cmnSavePosW(__WFILE__, __LINE__); cmnTraceErrW		// UNICODE verze
	#define THROWERRA	cmnSavePosA(__FILE__, __LINE__); cmnThrowErrA		// ANSI verze
	#define THROWERRW	cmnSavePosW(__WFILE__, __LINE__); cmnThrowErrW		// UNICODE verze

	// Define DBGTRACE, that writes only to the debugger window and only with DEBUG build
	#ifdef _DEBUG
      COMMON_API void dbgDebugTraceW(LPCWSTR lpszFormat, ...);	// UNICODE version
      COMMON_API void dbgDebugTraceA(LPCSTR lpszFormat, ...);	// ANSI version

	  // If the user wants to trace specific ANSI or UNICODE message
	  // These macros are always defined
	  #define DBGTRACEW	dbgDebugTraceW
	  #define DBGTRACEA	dbgDebugTraceA

      #ifdef UNICODE
        #define dbgDebugTrace	dbgDebugTraceW
      #else
        #define dbgDebugTrace	dbgDebugTraceA
      #endif

      #ifndef DBGTRACE
        #define DBGTRACE	dbgDebugTrace
      #endif
    #else	// _DEBUG isn't defined
      #ifndef DBGTRACE
	    #define DBGTRACE	__noop
      #endif
      #define DBGTRACEW	__noop
      #define DBGTRACEA	__noop
    #endif

    #ifdef UNICODE
		#define cmnTrace	TRACEW
		#define	cmnThrow	THROWW
		#define	cmnTraceErr	ERRW
		#define	cmnThrowErr	THROWERRW			
	#else
		#define cmnTrace	TRACEA
		#define	cmnThrow	THROWA
		#define	cmnTraceErr	ERRA
		#define	cmnThrowErr	THROWERRA	
    #endif
	//
	// Define own macros without MFC
	#define		TRACE						cmnTrace	
	#define		THROW						cmnThrow
	#define		THROWERR(str, hr)			cmnThrowErr(str, hr);	
	#define		ERR(str, hr)				cmnTraceErr(str, hr);
	//
	// Message box values - second parameter uType
	#define	MSG_ERROR		MB_ICONERROR
	#define	MSG_WARNING		MB_ICONWARNING
	#define	MSG_INFO		MB_ICONINFORMATION
	//
	// Message box titles
	#define	 _S_WARNINGW		L"Warning"
	#define	 _S_WARNINGA		"Warning"
	#define	 _S_ERRORW			L"Error"
	#define	 _S_ERRORA			"Error"
	#define	 _S_INFOW			L"Information"
	#define	 _S_INFOA			"Information"
	#define	 _S_COMMONW			L"Common message"	
	#define	 _S_COMMONA			"Common message"	
	#define	 UNKNOWN_DX_ERRW	L"Unknown"
	#define	 UNKNOWN_DX_ERRA	"Unknown"
	#define	 _S_LOG_TXTW		L"log.txt"
	#define	 _S_LOG_TXTA		"log.txt"

	#ifdef UNICODE
		#define	 _S_WARNING			_S_WARNINGW
		#define	 _S_ERROR			_S_ERRORW
		#define	 _S_INFO			_S_INFOW
		#define	 _S_COMMON			_S_COMMONW		
		#define	 UNKNOWN_DX_ERR		UNKNOWN_DX_ERRW
		#define	_S_LOG_TXT			_S_LOG_TXTW
	#else
		#define	 _S_WARNING			_S_WARNINGA
		#define	 _S_ERROR			_S_ERRORA
		#define	 _S_INFO			_S_INFOA
		#define	 _S_COMMON			_S_COMMONA	
		#define	 UNKNOWN_DX_ERR		UNKNOWN_DX_ERRA
		#define	_S_LOG_TXT			_S_LOG_TXTA
	#endif

	// The structure holds settings and other variables of the log module. The structure is available
	// directly only in Common.dll. Other modules have to access it via supplied functions.
	// The string variables are stored in UNICODE coding to avoid unnecessary conversions on WinNT kernel based 
	// Windows
	// Structure is initialized via constructor and cleanup operations are done in destructor
	typedef struct tagLogSettings {
		BOOL bUnicodeLogFile;
		BOOL bLogToFile;
		LPWSTR lpwszLogFile;

		HANDLE hLogFile;

		// Now the variables that store the line and filename where the error occured
		WCHAR wszFailFileName[MAX_PATH];
		DWORD dwFailLine;

		tagLogSettings();
		~tagLogSettings();
	} LOGSETTINGS, *LPLOGSETTINGS;

	// The 
	extern LOGSETTINGS g_sLogSettings;

	// The functions that makes the log settings structure accessible to the outer world
	inline COMMON_API BOOL cmnIsLogFileUnicode() { return g_sLogSettings.bUnicodeLogFile; }
	inline COMMON_API BOOL cmnIsLoggingToFile() { return g_sLogSettings.bLogToFile; }
	inline COMMON_API HANDLE cmnGetLogFileHandle() { return g_sLogSettings.hLogFile; }
	inline COMMON_API LPWSTR cmnGetFailFileName() { return g_sLogSettings.wszFailFileName; }
	inline COMMON_API DWORD cmnGetFailLineNumber() { return g_sLogSettings.dwFailLine; }

	COMMON_API DWORD cmnGetLogFileNameW(LPWSTR _lpwszBuffer, LPDWORD _lpdwSize);
	COMMON_API DWORD cmnGetLogFileNameA(LPSTR _lpszBuffer, LPDWORD _lpdwSize);

	#ifdef UNICODE
	  #define cmnGetLogFileName	cmnGetLogFileNameW
	#else
	  #define cmnGetLogFileName	cmnGetLogFileNameA
	#endif

	// Call to init the logger, otherwise you wan't be able to log to the file, if no file
	// is specified, then all logging goes to the debugger output only
	// The _bUnicodeLog value is used only when the filename is specified
	// Common.dll sets its default values as following (using constructor and destructor)
	//		bUnicodeLogFile = FALSE
	//		bLogToFile = FALSE
	//		lpwszLogFile = NULL
	//		hLogFile = NULL
	//		wszFileName[MAX_PATH] undefined
	//		dwLine = 0
	// Note: You cannot change the logfile name and its coding (UNICODE/ANSI) once the object is initialized
	COMMON_API DWORD cmnInitLogW(LPCWSTR _lpwszFile, BOOL _bUnicodeLog = TRUE);
	COMMON_API DWORD cmnInitLogA(LPCSTR _lpwszFile, BOOL _bUnicodeLog = TRUE);

	#ifdef UNICODE
	  #define cmnInitLog	cmnInitLogW
	#else
	  #define cmnInitLog	cmnInitLogA
	#endif
	
#endif // _CMNLOG_H_