// Tester.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "Tester.h"
#define MAX_LOADSTRING 100

#include "../common/common.h"
#include "../display/common.h"

XDisplay		g_theDisplay;
HWND			g_hWnd;

XMesh			g_Sphere;
XMesh			g_Box;
XMesh			g_Cylinder;
XMesh			g_Torus;
XMesh			g_Teapot;
XMesh			g_Tiger;
XMesh			g_Airplane;

XVertexBuffer	g_vbQuad;
XIndexBuffer	g_ibQuad;
XTexture		g_texMipmap;
BOOL			g_bQuadVis = FALSE;
BOOL			g_bQuadStop = FALSE;

// world matrix
D3DXMATRIX			g_matWorld1;
D3DXMATRIX			g_matWorld2;

D3DLIGHT8		g_PointLight;
D3DLIGHT8		g_Sun;

#define		_S_DEBUG			"Common"
#define		_S_TRACETOFILE		"TraceToFile"

void	UpdateFrame();
void	RestoreUserObjects();
void	FillBuffers();

// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// the main window class name

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
 	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_TESTER, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);
	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_TESTER);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	// inicializace logovaciho souboru, pokud je zapnuta funkce Tracetofile
	if(cmnReadSetupInt(_S_DEBUG, _S_TRACETOFILE) == 1) {
		cmnInitLog("log.txt", FALSE);
	}

	// init world matrices
	D3DXMatrixIdentity(&g_matWorld1);
	D3DXMatrixIdentity(&g_matWorld2);

	// inicializace Direct3D
	g_theDisplay.Init(g_hWnd);

	// preddefinovane objekty
	g_Sphere.CreateSphere(2.0f, 48, 48, g_theDisplay.GetDevice());
	g_Box.CreateBox(1.0f, 1.0f, 6.0f, g_theDisplay.GetDevice());
	g_Cylinder.CreateCylinder(1.5f, 2.5f, 4.0f, 24, 8, g_theDisplay.GetDevice());
	g_Torus.CreateTorus(1.0f, 2.0f, 64, 64, g_theDisplay.GetDevice());
	g_Teapot.CreateTeapot(g_theDisplay.GetDevice());
	// model tygra
	g_Tiger.LoadMeshFromFile("tiger.x", g_theDisplay.GetDevice(), g_theDisplay.GetResourceManager());
	g_Airplane.LoadMeshFromFile("airplane 2.x", g_theDisplay.GetDevice(), g_theDisplay.GetResourceManager());

	// zaregistrovani vsech zdroju
	g_theDisplay.GetResourceManager()->AddMesh(&g_Tiger);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Sphere);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Box);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Cylinder);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Teapot);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Torus);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Airplane);

	// prepare quad
	g_vbQuad.Create(4 * sizeof(VERTEX), D3DUSAGE_WRITEONLY, VERTEXFORMAT, D3DPOOL_DEFAULT, g_theDisplay.GetDevice());
	g_ibQuad.Create(6 * sizeof(WORD), D3DUSAGE_WRITEONLY, D3DFMT_INDEX16 , D3DPOOL_DEFAULT, g_theDisplay.GetDevice());
	g_texMipmap.LoadTextureFromFileEx("grass.bmp", 5, g_theDisplay.GetTextureFormat(), g_theDisplay.GetDevice());
	g_theDisplay.GetResourceManager()->AddIndexBuffer(&g_ibQuad);
	g_theDisplay.GetResourceManager()->AddVertexBuffer(&g_vbQuad);
	g_theDisplay.GetResourceManager()->AddTexture(&g_texMipmap);
	FillBuffers();
	
	// torus is visible by default
	g_Torus.Visible();

	// zde vytvorime slunicko
	ZeroMemory( &g_Sun, sizeof(D3DLIGHT8) );
	g_Sun.Type = D3DLIGHT_DIRECTIONAL;
	g_Sun.Direction.x = -1.0f;
	g_Sun.Direction.y = 0.0f;
	g_Sun.Direction.z = -1.0f;
	g_Sun.Diffuse.r  = 0.0f;
	g_Sun.Diffuse.g  = 0.0f;
	g_Sun.Diffuse.b  = 0.7f;
	g_Sun.Ambient.r  = 0.4f;
	g_Sun.Ambient.g  = 0.4f;
	g_Sun.Ambient.b  = 0.4f;
	g_Sun.Range = 1000.0f;	
	g_theDisplay.SetLight(0, &g_Sun);
	g_theDisplay.EnableLight(0, TRUE);

	// dalsi svetlo
	ZeroMemory( &g_PointLight, sizeof(D3DLIGHT8) );
	g_PointLight.Type = D3DLIGHT_POINT;
	g_PointLight.Position.x = 0.0f;
	g_PointLight.Position.y = 0.0f;
	g_PointLight.Position.z = 0.0f;
	g_PointLight.Diffuse.r  = 1.0f;
	g_PointLight.Diffuse.g  = 1.0f;
	g_PointLight.Diffuse.b  = 0.0f;
	g_PointLight.Range = 1000.0f;
	g_PointLight.Attenuation1 = 1.0f;	

	// Main message loop:
	while( TRUE )
    {
        // Look for messages, if none are found then 
        // update the state and display it
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( 0 == GetMessage(&msg, NULL, 0, 0 ) )
            {
                // WM_QUIT was posted, so exit
                return (int)msg.wParam;
            }
			
            // Translate and dispatch the message
            TranslateMessage( &msg ); 
            DispatchMessage( &msg );
        }
        else
        {
			// Obnova snimku
			UpdateFrame();			
		}
    }	
	return (int) msg.wParam;
}


void	FillBuffers()
{
	VERTEX *pVertices;
	g_vbQuad.GetBuffer()->Lock(0, 0, (BYTE**) &pVertices, 0);

	pVertices[0].vecPos = D3DXVECTOR3(-4.0f, -4.0f, 2.0f);
	pVertices[1].vecPos = D3DXVECTOR3(4.0f, -4.0f, 2.0f);
	pVertices[2].vecPos = D3DXVECTOR3(-4.0f, 4.0f, 2.0f);
	pVertices[3].vecPos = D3DXVECTOR3(4.0f, 4.0f, 2.0f);

	pVertices[0].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
	pVertices[1].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
	pVertices[2].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
	pVertices[3].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);

	pVertices[0].tu1 = 0.0f;
	pVertices[0].tv1 = 0.0f;

	pVertices[1].tu1 = 1.0f;
	pVertices[1].tv1 = 0.0f;

	pVertices[2].tu1 = 0.0f;
	pVertices[2].tv1 = 1.0f;

	pVertices[3].tu1 = 1.0f;
	pVertices[3].tv1 = 1.0f;

	pVertices[0].dwDiffuse = 0xFFFFFFFF;
	pVertices[1].dwDiffuse = 0xFFFFFFFF;
	pVertices[2].dwDiffuse = 0xFFFFFFFF;
	pVertices[3].dwDiffuse = 0xFFFFFFFF;

	g_vbQuad.GetBuffer()->Unlock();


	WORD *pIndices;
	g_ibQuad.GetBuffer()->Lock(0, 0, (BYTE**) &pIndices, 0);

	pIndices[0] = 0;
	pIndices[1] = 1;
	pIndices[2] = 2;

	pIndices[3] = 1;
	pIndices[4] = 3;
	pIndices[5] = 2;

	g_ibQuad.GetBuffer()->Unlock();
}
//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage are only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_TESTER);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{  
   hInst = hInstance; // Store instance handle in our global variable

   g_hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!g_hWnd)
   {
      return FALSE;
   }

   ShowWindow(g_hWnd, nCmdShow);
   UpdateWindow(g_hWnd);

   return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{	
	case WM_KEYUP:    
        switch( wParam )
        {
            case VK_F9:
				g_theDisplay.RestoreDisplay();
				RestoreUserObjects();
				FillBuffers();
				break;
			case VK_F1:
				g_Sphere.Visible(!g_Sphere.IsVisible());				
				g_bQuadVis = FALSE;
				break;
			case VK_F2:
				g_Torus.Visible(!g_Torus.IsVisible());				
				g_bQuadVis = FALSE;
				break;
			case VK_F3:
				g_Box.Visible(!g_Box.IsVisible());
				g_bQuadVis = FALSE;
				break;
			case VK_F4:
				g_Cylinder.Visible(!g_Cylinder.IsVisible());
				g_bQuadVis = FALSE;
				break;
			case VK_F5:
				g_Teapot.Visible(!g_Teapot.IsVisible());
				g_bQuadVis = FALSE;
				break;
			case VK_F6:
				g_Tiger.Visible(!g_Tiger.IsVisible());
				g_bQuadVis = FALSE;
				break;
			case VK_F7:
				g_Airplane.Visible(!g_Airplane.IsVisible());
				g_bQuadVis = FALSE;
				break;
			case VK_F8:
				g_Torus.Visible(FALSE);
				g_Sphere.Visible(FALSE);
				g_Box.Visible(FALSE);
				g_Tiger.Visible(FALSE);
				g_Airplane.Visible(FALSE);
				g_Teapot.Visible(FALSE);
				g_Cylinder.Visible(FALSE);
				g_bQuadVis = TRUE;
				break;
			case VK_SPACE:
				if(g_bQuadVis)
					g_bQuadStop = !g_bQuadStop;
				break;
			case VK_ESCAPE:
				PostQuitMessage(0);
				break;
		}
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}


void	UpdateFrame()
{
	// svetlo se bude pohybovat po kruznici
	g_PointLight.Position.x = 0.0f;
	g_PointLight.Position.y = float(4.0f * sin(double(GetTickCount())/300.0f));
	g_PointLight.Position.z = float(5.0f * cos(double(GetTickCount())/300.0f));
	g_theDisplay.SetLight(1, &g_PointLight);
	g_theDisplay.EnableLight(1, TRUE);
	// draw scene
	g_theDisplay.UpdateBackground();
	g_theDisplay.GetDevice()->BeginScene();				
	g_Sphere.Draw();
	g_Torus.Draw();
	g_Box.Draw();
	g_Cylinder.Draw();
	g_Teapot.Draw();
	// tygr nema definovane normalove vektory, takze je treba vypnout osvetleni
	g_theDisplay.GetDevice()->SetRenderState(D3DRS_LIGHTING, FALSE);
	g_Tiger.Draw();
	g_theDisplay.GetDevice()->SetRenderState(D3DRS_LIGHTING, TRUE);
	g_Airplane.Draw();

	if(g_bQuadVis)
	{
		D3DXMATRIX matRot;
		float fFactor = cmnGetTime(TIMER_GETELAPSEDTIME1);
		if(fFactor != 0  && !g_bQuadStop) {
			D3DXMatrixRotationZ(&matRot, 0.9f * fFactor);
			D3DXMatrixMultiply(&g_matWorld2, &g_matWorld2, &matRot);
			g_theDisplay.GetDevice()->SetTransform(D3DTS_WORLD, &g_matWorld2);	
		}	
		g_theDisplay.GetDevice()->SetRenderState(D3DRS_LIGHTING, FALSE);
		g_theDisplay.GetDevice()->SetVertexShader(VERTEXFORMAT);
		g_theDisplay.GetDevice()->SetStreamSource(0, g_vbQuad.GetBuffer(), sizeof(VERTEX));
		g_theDisplay.GetDevice()->SetIndices(g_ibQuad.GetBuffer(), 0);
		g_theDisplay.GetDevice()->SetTexture(0, g_texMipmap.GetTexture());
		g_theDisplay.GetDevice()->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 4, 0, 2);
		g_theDisplay.GetDevice()->SetRenderState(D3DRS_LIGHTING, TRUE);
	}
	else
	{
		D3DXMATRIX matRot;
		float fFactor = cmnGetTime(TIMER_GETELAPSEDTIME1);
		if(fFactor != 0) {
			D3DXMatrixRotationX(&matRot, 0.9f * fFactor);
			D3DXMatrixMultiply(&g_matWorld1, &g_matWorld1, &matRot);
			g_theDisplay.GetDevice()->SetTransform(D3DTS_WORLD, &g_matWorld1);	
		}	
	}

	g_theDisplay.GetDevice()->EndScene();
	if(D3DERR_DEVICENOTRESET == g_theDisplay.Present())
	{
		RestoreUserObjects();
		FillBuffers();
	}
}

void	RestoreUserObjects()
{
	g_theDisplay.SetLight(0, &g_Sun);
	g_theDisplay.EnableLight(0, TRUE);
}